﻿// CameraController.cs
// by Dickson Cho
// August 2016
using UnityEngine;
using System.Collections;

enum CameraState
{
	eFlyingIntro,
	eWaitingIntro,
	eInGame,

	MAX_STATES
}


/// <summary>
/// Camera controller to give the game that smooth camera angle. This
/// is what allows the camera to zoom around, pan, do cinematic stuff, etc.
/// This is also the place where we would need to implement what happens
/// when the camera collides with a wall or other mesh in the world. This
/// script needs to be placed in the camera rig
/// </summary>
public class CameraController : MonoBehaviour
{
	public GameObject m_FollowObject;			// The player object to follow
	public float m_RotationSpeed;               // How fast to rotate the camera, in degrees per second

	private CameraState m_State;				// What the camera is doing at the moment

	#region Specific to Intro Camera
	public GameObject[] m_IntroCameraPositions;	// List of positions the camera should fly by
	public float m_IntroCameraWaitForSeconds;	// Number of seconds to wait between each point
	public float m_IntroCameraSecondsPerTrip;	// Speed at which the camera flies

	private int m_CurrentIntroCameraPosition;   // The current position we're flying away from
	private float m_CurrentIntroCameraWait;		// How long we've been waiting at the current position
	private float m_CurrentIntroCameraLerp;		// Our lerp progress to the next camera position
	#endregion

	private void Start()
	{
		if (m_IntroCameraPositions.Length == 0)
		{
			// No intro to fly around on
			m_State = CameraState.eInGame;
		}
		else
		{
			// Get us set up for the flying intro, once game state tells us to go
			m_State = CameraState.eFlyingIntro;
			transform.position = m_IntroCameraPositions[0].transform.position;
			transform.rotation = m_IntroCameraPositions[0].transform.rotation;

			m_CurrentIntroCameraPosition = 0;
			m_CurrentIntroCameraWait = m_IntroCameraWaitForSeconds;
			m_CurrentIntroCameraLerp = 0.0f;
		}
	}

	private void Update()
	{
		GameState state = GameManager.GetInstance().m_State;
		switch(state)
		{
			case GameState.eFlyingIntroCamera:
				UpdateIntroCamera();
				break;
			case GameState.eInGame:
				// Keep it simple for now. Do nice tweens later
				// DICKSON TODO: Work on camera clipping through the walls
				transform.position = m_FollowObject.transform.position;
				break;
		}
	}

	/// <summary>
	/// Rotate the camera around the target object
	/// </summary>
	public void RotateLeft()
	{
		float degrees = m_RotationSpeed * Time.deltaTime * -1;
		gameObject.transform.Rotate(Vector3.up, degrees);
	}

	/// <summary>
	/// Rotate the camera around the target object
	/// </summary>
	public void RotateRight()
	{
		float degrees = m_RotationSpeed * Time.deltaTime;
		gameObject.transform.Rotate(Vector3.up, degrees);
	}

	/// <summary>
	/// For the impatient, this function will skip all camera cuts and go straight to the final camera
	/// (minus the first camera cut)
	/// </summary>
	public void SkipIntro()
	{
		if (m_CurrentIntroCameraPosition > 0)
		{
			transform.position = m_FollowObject.transform.position;
			transform.rotation = Quaternion.identity;
			m_State = CameraState.eInGame;
			GameManager.GetInstance().AdvanceGameState();
		}
	}

	/// <summary>
	/// Special function to take care of the neat intro fly-by
	/// </summary>
	private void UpdateIntroCamera()
	{
		switch (m_State)
		{
			case CameraState.eFlyingIntro:
				// First, establish the start and end points
				Vector3 startPos = m_IntroCameraPositions[m_CurrentIntroCameraPosition].transform.position;
				Quaternion startRot = m_IntroCameraPositions[m_CurrentIntroCameraPosition].transform.rotation;
				Vector3 endPos;
				Quaternion endRot;
				if (m_CurrentIntroCameraPosition < m_IntroCameraPositions.Length - 1)
				{
					endPos = m_IntroCameraPositions[m_CurrentIntroCameraPosition + 1].transform.position;
					endRot = m_IntroCameraPositions[m_CurrentIntroCameraPosition + 1].transform.rotation;
				}
				else
				{
					endPos = m_FollowObject.transform.position;
					endRot = Quaternion.identity;
				}

				// Now, lerp from start to end
				m_CurrentIntroCameraLerp += Time.deltaTime / m_IntroCameraSecondsPerTrip;
				transform.position = Vector3.Lerp(startPos, endPos, m_CurrentIntroCameraLerp);
				transform.rotation = Quaternion.Lerp(startRot, endRot, m_CurrentIntroCameraLerp);
				if (m_CurrentIntroCameraLerp >= 1.0f)
				{
					m_State = CameraState.eWaitingIntro;
					m_CurrentIntroCameraLerp = 0.0f;
					m_CurrentIntroCameraPosition++;
				}
				break;
			case CameraState.eWaitingIntro:
				m_CurrentIntroCameraWait -= Time.deltaTime;
				if (m_CurrentIntroCameraWait <= 0.0f)
				{
					m_CurrentIntroCameraWait = m_IntroCameraWaitForSeconds;

					// Are we at the last point?
					if (m_CurrentIntroCameraPosition < m_IntroCameraPositions.Length)
					{
						m_State = CameraState.eFlyingIntro;
					}
					else
					{
						m_State = CameraState.eInGame;
					}
				}
				break;
			case CameraState.eInGame:
				GameManager.GetInstance().AdvanceGameState();
				break;
			default:
				break;
		}
	}
}
